<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use Cake\Core\Configure;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Association\HasMany;
use Cake\ORM\Behavior\TreeBehavior;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ProductCategory;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ProductCategories Model
 *
 * @property ProductCatalogsTable&BelongsTo $ProductCatalogs
 * @property ProductCategoriesTable&BelongsTo $ParentProductCategories
 * @property ProductCategoriesTable&HasMany $ChildProductCategories
 *
 * @method ProductCategory newEmptyEntity()
 * @method ProductCategory newEntity(array $data, array $options = [])
 * @method array<ProductCategory> newEntities(array $data, array $options = [])
 * @method ProductCategory get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ProductCategory findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ProductCategory patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ProductCategory> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ProductCategory saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ProductCategory>|ResultSetInterface<ProductCategory>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategory>|ResultSetInterface<ProductCategory> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ProductCategory>|ResultSetInterface<ProductCategory>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategory>|ResultSetInterface<ProductCategory> deleteManyOrFail(iterable $entities, array $options = [])
 *
 * @mixin TreeBehavior
 */
class ProductCategoriesTable extends Table
{
    /**
     * Current scope for Tree behavior - per catalog
     *
     * @var string
     */
    protected $treeCatalogId;

    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);
        $this->treeCatalogId = 1;

        $this->setTable('product_categories');
        $this->setDisplayField('name');
        $this->setPrimaryKey('id');
        $this->setEntityClass(
            Configure::read('CakeProducts.ProductCategories.entity', 'CakeProducts\Model\Entity\ProductCategory')
        );
        $this->addBehavior('Tree');

        $this->belongsTo('ProductCatalogs', [
            'foreignKey' => 'product_catalog_id',
            'joinType' => 'INNER',
            'className' => 'CakeProducts.ProductCatalogs',
        ]);
        $this->belongsTo('ParentProductCategories', [
            'className' => 'CakeProducts.ProductCategories',
            'foreignKey' => 'parent_id',
        ]);
        $this->hasMany('ChildProductCategories', [
            'className' => 'CakeProducts.ProductCategories',
            'foreignKey' => 'parent_id',
        ]);
        $this->hasMany('ProductCategoryAttributes', [
            'foreignKey' => 'product_category_id',
            'bindingKey' => 'internal_id',
            'className' => 'CakeProducts.ProductCategoryAttributes',
        ]);
        $this->behaviors()->Tree->setConfig('scope', ['product_catalog_id' => $this->treeCatalogId]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->uuid('product_catalog_id')
            ->notEmptyString('product_catalog_id');

        $validator
            ->scalar('name')
            ->maxLength('name', 255)
            ->requirePresence('name', 'create')
            ->notEmptyString('name');

        $validator
            ->scalar('category_description')
            ->allowEmptyString('category_description');

        $validator
            ->integer('parent_id')
            ->allowEmptyString('parent_id');

        $validator
            ->boolean('enabled')
            ->notEmptyString('enabled');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->isUnique(['product_catalog_id', 'name']), ['errorField' => 'product_catalog_id']);
        $rules->add($rules->existsIn(['product_catalog_id'], 'ProductCatalogs'), ['errorField' => 'product_catalog_id']);
        $rules->add($rules->existsIn(['parent_id'], 'ParentProductCategories'), ['errorField' => 'parent_id']);

        return $rules;
    }

    /**
     * @param int $catalogId
     *
     * @return void
     */
    public function setConfigureCatalogId(string $catalogId)
    {
        $this->treeCatalogId = $catalogId;
        $this->behaviors()->Tree->setConfig('scope', ['product_catalog_id' => $this->treeCatalogId]);
    }

    /**
     * @param EntityInterface $entity
     * @param array $options
     *
     * @return EntityInterface|false
     */
    public function save(EntityInterface $entity, array $options = []): EntityInterface|false
    {
        $this->behaviors()->get('Tree')->setConfig([
            'scope' => [
                'product_catalog_id' => $entity->product_catalog_id,
            ],
        ]);

        return parent::save($entity, $options);
    }
}
