<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use Cake\Database\Type\EnumType;
use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ProductCategoryAttribute;
use CakeProducts\Model\Enum\ProductCategoryAttributeTypeId;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ProductCategoryAttributes Model
 *
 * @property ProductCategoriesTable&BelongsTo $ProductCategories
 *
 * @method ProductCategoryAttribute newEmptyEntity()
 * @method ProductCategoryAttribute newEntity(array $data, array $options = [])
 * @method array<ProductCategoryAttribute> newEntities(array $data, array $options = [])
 * @method ProductCategoryAttribute get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ProductCategoryAttribute findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ProductCategoryAttribute patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ProductCategoryAttribute> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ProductCategoryAttribute|false save(EntityInterface $entity, array $options = [])
 * @method ProductCategoryAttribute saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ProductCategoryAttribute>|ResultSetInterface<ProductCategoryAttribute>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttribute>|ResultSetInterface<ProductCategoryAttribute> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttribute>|ResultSetInterface<ProductCategoryAttribute>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ProductCategoryAttribute>|ResultSetInterface<ProductCategoryAttribute> deleteManyOrFail(iterable $entities, array $options = [])
 */
class ProductCategoryAttributesTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('product_category_attributes');
        $this->setDisplayField('name');
        $this->setPrimaryKey('id');

        $this->belongsTo('ProductCategories', [
            'foreignKey' => 'product_category_id',
            'bindingKey' => 'internal_id',
            'className' => 'CakeProducts.ProductCategories',
        ]);

        $this->hasMany('ProductCategoryAttributeOptions', [
            'foreignKey' => 'product_category_attribute_id',
            'className' => 'CakeProducts.ProductCategoryAttributeOptions',
            'saveStrategy' => 'replace',
        ]);
        $this->getSchema()->setColumnType('attribute_type_id', EnumType::from(ProductCategoryAttributeTypeId::class));
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->scalar('name')
            ->maxLength('name', 255)
            ->requirePresence('name', 'create')
            ->notEmptyString('name');

        $validator
            ->uuid('product_category_id')
            ->allowEmptyString('product_category_id');

        $validator
            ->integer('attribute_type_id')
            ->requirePresence('attribute_type_id', 'create')
            ->notEmptyString('attribute_type_id');

        $validator
            ->boolean('enabled')
            ->requirePresence('enabled', 'create')
            ->notEmptyString('enabled');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->isUnique(['name', 'product_category_id'], ['allowMultipleNulls' => true]), ['errorField' => 'name']);
        $rules->add($rules->existsIn(['product_category_id'], 'ProductCategories'), ['errorField' => 'product_category_id']);

        return $rules;
    }
}
