<?php
declare(strict_types=1);

namespace CakeProducts\Test\TestCase\Controller;

use Cake\ORM\Table;
use Cake\TestSuite\IntegrationTestTrait;
use Cake\TestSuite\TestCase;
use CakeProducts\Controller\ProductCatalogsController;
use CakeProducts\Model\Table\ProductCatalogsTable;
use PHPUnit\Exception;

/**
 * CakeProducts\Controller\ProductCatalogsController Test Case
 *
 * @uses \CakeProducts\Controller\ProductCatalogsController
 */
class ProductCatalogsControllerTest extends BaseControllerTest
{
    /**
     * Test subject table
     *
     * @var ProductCatalogsTable|Table
     */
    protected $ProductCatalogs;

    /**
     * Fixtures
     *
     * @var array<string>
     */
    protected array $fixtures = [
        'plugin.CakeProducts.ProductCatalogs',
        'plugin.CakeProducts.ProductCategories',
    ];

    /**
     * setUp method
     *
     * @return void
     */
    protected function setUp(): void
    {
        parent::setUp();
//        $this->enableCsrfToken();
//        $this->enableSecurityToken();
        $this->disableErrorHandlerMiddleware();
        $config = $this->getTableLocator()->exists('ProductCatalogs') ? [] : ['className' => ProductCatalogsTable::class];
        $this->ProductCatalogs = $this->getTableLocator()->get('ProductCatalogs', $config);
    }

    /**
     * tearDown method
     *
     * @return void
     */
    protected function tearDown(): void
    {
        unset($this->ProductCatalogs);

        parent::tearDown();
    }

    /**
    * Test index method
    *
    * Tests the index action with a logged in user
    *
    * @uses \CakeProducts\Controller\ProductCatalogsController::index()
    * @throws Exception
    *
    * @return void
    */
    public function testIndexGet(): void
    {
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'index',
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test view method
    *
    * Tests the view action with a logged in user
    *
    * @uses ProductCatalogsController::view()
    * @throws Exception
    *
    * @return void
    */
    public function testViewGet(): void
    {
        $id = '115153f3-2f59-4234-8ff8-e1b205761428';
        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'view',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test add method
    *
    * Tests the add action with a logged in user
    *
    * @uses ProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddGet(): void
    {
        $cntBefore = $this->ProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'add',
        ];
        $this->get($url);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCatalogs->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses ProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostSuccess(): void
    {
        $cntBefore = $this->ProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'add',
        ];
        $data = [
            'name' => 'new catalog',
            'catalog_description' => 'description',
            'enabled' => true,
        ];
        $this->post($url, $data);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-catalogs');

        $cntAfter = $this->ProductCatalogs->find()->count();
        $this->assertEquals($cntBefore + 1, $cntAfter);
    }

    /**
    * Test add method
    *
    * Tests a POST request to the add action with a logged in user
    *
    * @uses ProductCatalogsController::add()
    * @throws Exception
    *
    * @return void
    */
    public function testAddPostFailure(): void
    {
        $cntBefore = $this->ProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'add',
        ];
        $data = [
            'name' => '',
            'catalog_description' => '',
            'enabled' => '',
        ];
        $this->post($url, $data);
        $this->assertResponseCode(200);

        $cntAfter = $this->ProductCatalogs->find()->count();
        $this->assertEquals($cntBefore, $cntAfter);
    }

    /**
    * Test edit method
    *
    * Tests the edit action with a logged in user
    *
    * @uses ProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditGet(): void
    {
        $this->loginUserByRole('admin');
        $id = '115153f3-2f59-4234-8ff8-e1b205761428';
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'edit',
            $id,
        ];
        $this->get($url);
        $this->assertResponseCode(200);
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses ProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutSuccess(): void
    {
        $this->loginUserByRole('admin');
        $id = '115153f3-2f59-4234-8ff8-e1b205761428';
//        $before = $this->ProductCatalogs->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'edit',
            $id,
        ];
        $data = [
            // test new data here
            'name' => 'edited name',
            'catalog_description' => 'new catalog description',
            'enabled' => true,
        ];
        $this->put($url, $data);

        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-catalogs');

        $after = $this->ProductCatalogs->get($id);
        $this->assertEquals($data['name'], $after->name);
        $this->assertEquals($data['catalog_description'], $after->catalog_description);
        // assert saved properly below
    }

    /**
    * Test edit method
    *
    * Tests a PUT request to the edit action with a logged in user
    *
    * @uses ProductCatalogsController::edit()
    * @throws Exception
    *
    * @return void
    */
    public function testEditPutFailure(): void
    {
        $this->loginUserByRole('admin');
        $id = '115153f3-2f59-4234-8ff8-e1b205761428';
        $before = $this->ProductCatalogs->get($id);
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'edit',
            $id,
        ];
        $data = [
            'name' => '',
            'catalog_description' => 'edited description',
            'enabled' => '',
        ];
        $this->put($url, $data);
        $this->assertResponseCode(200);
        $after = $this->ProductCatalogs->get($id);
        $this->assertEquals($before->name, $after->name);
        $this->assertEquals($before->catalog_description, $after->catalog_description);
        // assert save failed below
    }

    /**
    * Test delete method
    *
    * Tests the delete action with a logged in user
    *
    * @uses ProductCatalogsController::delete()
    * @throws Exception
    *
    * @return void
    */
    public function testDelete(): void
    {
        $cntBefore = $this->ProductCatalogs->find()->count();

        $this->loginUserByRole('admin');
        $id = '115153f3-2f59-4234-8ff8-e1b205761428';
        $url = [
            'plugin' => 'CakeProducts',
            'controller' => 'ProductCatalogs',
            'action' => 'delete',
            $id,
        ];
        $this->delete($url);
        $this->assertResponseCode(302);
        $this->assertRedirectContains('product-catalogs');

        $cntAfter = $this->ProductCatalogs->find()->count();
        $this->assertEquals($cntBefore - 1, $cntAfter);
    }
}
